(xsl:transform
  ((xmlns:xsl
     .
     "http://www.w3.org/1999/XSL/Transform")
   (version . "1.0")
   (xmlns:e . "http://exslt.org/common")
   (xmlns:func . "http://exslt.org/functions")
   (xmlns:str . "http://exslt.org/strings")
   (xmlns:xcb . "http://xcb.freedesktop.org")
   (extension-element-prefixes . "func str xcb"))
  (xsl:output ((method . "text")))
  (xsl:strip-space ((elements . "*")))
  (xsl:param ((name . "mode")))
  (xsl:param ((name . "base-path")))
  (xsl:param
    ((name . "extension-path")
     (select . "$base-path")))
  (xsl:variable
    ((name . "h") (select . "$mode = 'header'")))
  (xsl:variable
    ((name . "c") (select . "$mode = 'source'")))
  (xsl:variable
    ((name . "need-string-h")
     (select . "//request/pad[@bytes != 1]")))
  (xsl:variable
    ((name . "indent-string") (select . "'    '")))
  (xsl:variable
    ((name . "ucase")
     (select . "'ABCDEFGHIJKLMNOPQRSTUVWXYZ'")))
  (xsl:variable
    ((name . "lcase")
     (select . "'abcdefghijklmnopqrstuvwxyz'")))
  (xsl:variable
    ((name . "letters")
     (select . "concat($ucase, $lcase)")))
  (xsl:variable
    ((name . "digits") (select . "'0123456789'")))
  (xsl:variable
    ((name . "header") (select . "/xcb/@header")))
  (xsl:variable
    ((name . "ucase-header")
     (select . "translate($header,$lcase,$ucase)")))
  (xsl:variable
    ((name . "ext")
     (select . "/xcb/@extension-name")))
  (xsl:variable
    ((name . "search-path-rtf"))
    (xsl:for-each
      ((select . "/xcb/import"))
      (path ()
            (xsl:value-of
              ((select . "concat($extension-path, ., '.xml')"))))))
  (xsl:variable
    ((name . "search-path")
     (select . "e:node-set($search-path-rtf)/path")))
  (xsl:variable ((name . "root") (select . "/")))
  (xsl:variable
    ((name . "pass1-rtf"))
    (xsl:apply-templates
      ((select . "/") (mode . "pass1"))))
  (xsl:variable
    ((name . "pass1")
     (select . "e:node-set($pass1-rtf)")))
  (xsl:template
    ((match . "xcb") (mode . "pass1"))
    (xcb ()
         (xsl:copy-of ((select . "@*")))
         (xsl:if
           ((test . "$ext"))
           (constant
             ((type . "xcb_extension_t")
              (name . "{xcb:xcb-prefix()}_id"))
             (xsl:attribute
               ((name . "value"))
               "{ \""
               (xsl:value-of ((select . "@extension-xname")))
               "\" }")))
         (xsl:apply-templates ((mode . "pass1")))))
  (func:function
    ((name . "xcb:xcb-prefix"))
    (xsl:param ((name . "name")))
    (func:result
      ()
      (xsl:text () "xcb")
      (xsl:choose
        ()
        (xsl:when
          ((test . "/xcb/@extension-name = 'RandR'"))
          (xsl:text () "_randr"))
        (xsl:when
          ((test . "/xcb/@extension-name = 'ScreenSaver'"))
          (xsl:text () "_screensaver"))
        (xsl:when
          ((test . "/xcb/@extension-name = 'XF86Dri'"))
          (xsl:text () "_xf86dri"))
        (xsl:when
          ((test . "/xcb/@extension-name = 'XFixes'"))
          (xsl:text () "_xfixes"))
        (xsl:when
          ((test . "/xcb/@extension-name = 'XvMC'"))
          (xsl:text () "_xvmc"))
        (xsl:when
          ((test . "/xcb/@extension-name"))
          (xsl:text () "_")
          (xsl:call-template
            ((name . "camelcase-to-underscore"))
            (xsl:with-param
              ((name . "camelcase")
               (select . "/xcb/@extension-name")))))
        (xsl:otherwise ()))
      (xsl:if
        ((test . "$name"))
        (xsl:text () "_")
        (xsl:call-template
          ((name . "camelcase-to-underscore"))
          (xsl:with-param
            ((name . "camelcase") (select . "$name")))))))
  (func:function
    ((name . "xcb:lowercase"))
    (xsl:param ((name . "name")))
    (func:result
      ()
      (xsl:call-template
        ((name . "camelcase-to-underscore"))
        (xsl:with-param
          ((name . "camelcase") (select . "$name"))))))
  (func:function
    ((name . "xcb:get-char-void"))
    (xsl:param ((name . "name")))
    (xsl:variable
      ((name . "ctype")
       (select . "substring-before($name, '_t')")))
    (func:result
      ()
      (xsl:choose
        ()
        (xsl:when
          ((test .
                 "$ctype = 'char' or $ctype = 'void' or $ctype = 'float' or $ctype = 'double'"))
          (xsl:value-of ((select . "$ctype"))))
        (xsl:otherwise
          ()
          (xsl:value-of ((select . "$name")))))))
  (func:function
    ((name . "xcb:remove-void"))
    (xsl:param ((name . "name")))
    (xsl:variable
      ((name . "ctype")
       (select . "substring-before($name, '_t')")))
    (func:result
      ()
      (xsl:choose
        ()
        (xsl:when
          ((test .
                 "$ctype = 'char' or $ctype = 'void' or $ctype = 'float' or $ctype = 'double'"))
          (xsl:choose
            ()
            (xsl:when
              ((test . "$ctype = 'void'"))
              (xsl:text () "char"))
            (xsl:otherwise
              ()
              (xsl:value-of ((select . "$ctype"))))))
        (xsl:otherwise
          ()
          (xsl:value-of ((select . "$name")))))))
  (xsl:template
    ((name . "camelcase-to-underscore"))
    (xsl:param ((name . "camelcase")))
    (xsl:choose
      ()
      (xsl:when
        ((test .
               "$camelcase='CHAR2B' or $camelcase='INT64'                       or $camelcase='FLOAT32' or $camelcase='FLOAT64'                       or $camelcase='BOOL32' or $camelcase='STRING8'                       or $camelcase='Family_DECnet'"))
        (xsl:value-of
          ((select
             .
             "translate($camelcase, $ucase, $lcase)"))))
      (xsl:otherwise
        ()
        (xsl:for-each
          ((select . "str:split($camelcase, '')"))
          (xsl:variable ((name . "a") (select . ".")))
          (xsl:variable
            ((name . "b") (select . "following::*[1]")))
          (xsl:variable
            ((name . "c") (select . "following::*[2]")))
          (xsl:value-of
            ((select . "translate(., $ucase, $lcase)")))
          (xsl:if
            ((test .
                   "($b and contains($lcase, $a) and contains($ucase, $b))                         or ($b and contains($digits, $a)                             and contains($letters, $b))                         or ($b and contains($letters, $a)                             and contains($digits, $b))                         or ($c and contains($ucase, $a)                             and contains($ucase, $b)                             and contains($lcase, $c))"))
            (xsl:text () "_"))))))
  (xsl:template
    ((name . "canonical-var-name"))
    (xsl:param ((name . "name") (select . "@name")))
    (xsl:if
      ((test .
             "$name='new' or $name='delete'                   or $name='class' or $name='operator'"))
      (xsl:text () "_"))
    (xsl:value-of ((select . "$name"))))
  (xsl:variable
    ((name . "core-types-rtf"))
    (type ((name . "BOOL") (newname . "uint8")))
    (type ((name . "BYTE") (newname . "uint8")))
    (type ((name . "CARD8") (newname . "uint8")))
    (type ((name . "CARD16") (newname . "uint16")))
    (type ((name . "CARD32") (newname . "uint32")))
    (type ((name . "INT8") (newname . "int8")))
    (type ((name . "INT16") (newname . "int16")))
    (type ((name . "INT32") (newname . "int32")))
    (type ((name . "char") (newname . "char")))
    (type ((name . "void") (newname . "void")))
    (type ((name . "float") (newname . "float")))
    (type ((name . "double") (newname . "double"))))
  (xsl:variable
    ((name . "core-types")
     (select . "e:node-set($core-types-rtf)")))
  (xsl:template
    ((name . "canonical-type-name"))
    (xsl:param
      ((name . "type") (select . "string(@type)")))
    (xsl:variable
      ((name . "is-unqualified")
       (select . "not(contains($type, ':'))")))
    (xsl:variable
      ((name . "namespace")
       (select . "substring-before($type, ':')")))
    (xsl:variable
      ((name . "unqualified-type"))
      (xsl:choose
        ()
        (xsl:when
          ((test . "$is-unqualified"))
          (xsl:value-of ((select . "$type"))))
        (xsl:otherwise
          ()
          (xsl:value-of
            ((select . "substring-after($type, ':')"))))))
    (xsl:choose
      ()
      (xsl:when
        ((test .
               "$is-unqualified and $core-types/type[@name=$type]"))
        (xsl:value-of
          ((select
             .
             "$core-types/type[@name=$type]/@newname"))))
      (xsl:otherwise
        ()
        (xsl:variable
          ((name . "type-definitions")
           (select
             .
             "(/xcb|document($search-path)/xcb                               )[$is-unqualified or @header=$namespace]                                /*[((self::struct or self::union or self::enum                                     or self::xidtype or self::xidunion                                     or self::event or self::eventcopy                                     or self::error or self::errorcopy)                                    and @name=$unqualified-type)                                   or (self::typedef                                       and @newname=$unqualified-type)]")))
        (xsl:choose
          ()
          (xsl:when
            ((test . "count($type-definitions) = 1"))
            (xsl:for-each
              ((select . "$type-definitions"))
              (xsl:value-of
                ((select . "xcb:xcb-prefix($unqualified-type)")))))
          (xsl:when
            ((test . "count($type-definitions) > 1"))
            (xsl:message
              ((terminate . "yes"))
              (xsl:text () "Multiple definitions of type \"")
              (xsl:value-of ((select . "$type")))
              (xsl:text () "\" found.")
              (xsl:if
                ((test . "$is-unqualified"))
                (xsl:for-each
                  ((select . "$type-definitions"))
                  (xsl:text ())
                  (xsl:value-of
                    ((select . "concat(/xcb/@header, ':', $type)")))))))
          (xsl:otherwise
            ()
            (xsl:message
              ((terminate . "yes"))
              (xsl:text () "No definitions of type \"")
              (xsl:value-of ((select . "$type")))
              (xsl:text () "\" found")
              (xsl:if
                ((test . "$is-unqualified"))
                (xsl:text () ", and it is not a known core type"))
              (xsl:text () ".")))))))
  (xsl:template
    ((name . "cookie-type"))
    (xsl:param ((name . "request") (select . ".")))
    (xsl:choose
      ()
      (xsl:when
        ((test . "$request/reply"))
        (xsl:value-of
          ((select . "xcb:xcb-prefix($request/@name)"))))
      (xsl:otherwise () (xsl:text () "xcb_void")))
    (xsl:text () "_cookie_t"))
  (xsl:template
    ((name . "request-function"))
    (xsl:param ((name . "checked")))
    (xsl:param ((name . "req")))
    (function
      ()
      (xsl:attribute
        ((name . "name"))
        (xsl:value-of
          ((select . "xcb:xcb-prefix($req/@name)")))
        (xsl:if
          ((test . "$checked='true' and not($req/reply)"))
          "_checked")
        (xsl:if
          ((test . "$checked='false' and $req/reply"))
          "_unchecked"))
      (xsl:attribute
        ((name . "type"))
        (xsl:call-template
          ((name . "cookie-type"))
          (xsl:with-param
            ((name . "request") (select . "$req")))))
      (field ((type . "xcb_connection_t *") (name . "c")))
      (xsl:apply-templates
        ((select . "$req/*[not(self::reply)]")
         (mode . "param")))
      (do-request
        ((ref . "{xcb:xcb-prefix($req/@name)}_request_t")
         (opcode
           .
           "{translate(xcb:xcb-prefix($req/@name), $lcase, $ucase)}")
         (checked . "{$checked}"))
        (xsl:if
          ((test . "$req/reply"))
          (xsl:attribute ((name . "has-reply")) "true")))))
  (xsl:template
    ((match . "request") (mode . "pass1"))
    (xsl:variable ((name . "req") (select . ".")))
    (xsl:if
      ((test . "reply"))
      (struct
        ((name . "{xcb:xcb-prefix(@name)}_cookie_t"))
        (field ((type . "unsigned int") (name . "sequence")))))
    (constant
      ((type . "number")
       (name . "{xcb:xcb-prefix($req/@name)}")
       (value . "{$req/@opcode}")))
    (struct
      ((name . "{xcb:xcb-prefix(@name)}_request_t"))
      (field ((type . "uint8_t")
              (name . "major_opcode")
              (no-assign . "true")))
      (xsl:if
        ((test . "$ext"))
        (field ((type . "uint8_t")
                (name . "minor_opcode")
                (no-assign . "true"))))
      (xsl:apply-templates
        ((select . "*[not(self::reply)]")
         (mode . "field")))
      (middle
        ()
        (field ((type . "uint16_t")
                (name . "length")
                (no-assign . "true")))))
    (xsl:call-template
      ((name . "request-function"))
      (xsl:with-param
        ((name . "checked") (select . "'true'")))
      (xsl:with-param
        ((name . "req") (select . "$req"))))
    (xsl:call-template
      ((name . "request-function"))
      (xsl:with-param
        ((name . "checked") (select . "'false'")))
      (xsl:with-param
        ((name . "req") (select . "$req"))))
    (xsl:if
      ((test . "reply"))
      (struct
        ((name . "{xcb:xcb-prefix(@name)}_reply_t"))
        (field ((type . "uint8_t") (name . "response_type")))
        (xsl:apply-templates
          ((select . "reply/*") (mode . "field")))
        (middle
          ()
          (field ((type . "uint16_t") (name . "sequence")))
          (field ((type . "uint32_t") (name . "length")))))
      (iterator-functions
        ((ref . "{xcb:xcb-prefix(@name)}")
         (kind . "_reply")))
      (function
        ((type . "{xcb:xcb-prefix(@name)}_reply_t *")
         (name . "{xcb:xcb-prefix(@name)}_reply"))
        (field ((type . "xcb_connection_t *") (name . "c")))
        (field ((name . "cookie"))
               (xsl:attribute
                 ((name . "type"))
                 (xsl:call-template ((name . "cookie-type")))))
        (field ((type . "xcb_generic_error_t **") (name . "e")))
        (l ()
           "return ("
           (xsl:value-of
             ((select . "xcb:xcb-prefix(@name)")))
           "_reply_t *)"
           " xcb_wait_for_reply(c, cookie.sequence, e);"))))
  (xsl:template
    ((match . "xidtype|xidunion") (mode . "pass1"))
    (typedef
      ((oldname . "uint32_t")
       (newname . "{xcb:xcb-prefix(@name)}_t")))
    (iterator ((ref . "{xcb:xcb-prefix(@name)}")))
    (iterator-functions
      ((ref . "{xcb:xcb-prefix(@name)}"))))
  (xsl:template
    ((match . "struct|union") (mode . "pass1"))
    (struct
      ((name . "{xcb:xcb-prefix(@name)}_t"))
      (xsl:if
        ((test . "self::union"))
        (xsl:attribute ((name . "kind")) "union"))
      (xsl:apply-templates
        ((select . "*") (mode . "field"))))
    (iterator ((ref . "{xcb:xcb-prefix(@name)}")))
    (iterator-functions
      ((ref . "{xcb:xcb-prefix(@name)}"))))
  (xsl:template
    ((match . "event|eventcopy|error|errorcopy")
     (mode . "pass1"))
    (xsl:variable
      ((name . "suffix"))
      (xsl:choose
        ()
        (xsl:when
          ((test . "self::event|self::eventcopy"))
          (xsl:text () "_event_t"))
        (xsl:when
          ((test . "self::error|self::errorcopy"))
          (xsl:text () "_error_t"))))
    (constant
      ((type . "number")
       (name . "{xcb:xcb-prefix(@name)}")
       (value . "{@number}")))
    (xsl:choose
      ()
      (xsl:when
        ((test . "self::event|self::error"))
        (struct
          ((name . "{xcb:xcb-prefix(@name)}{$suffix}"))
          (field ((type . "uint8_t") (name . "response_type")))
          (xsl:if
            ((test . "self::error"))
            (field ((type . "uint8_t") (name . "error_code"))))
          (xsl:apply-templates
            ((select . "*") (mode . "field")))
          (xsl:if
            ((test .
                   "not(self::event and boolean(@no-sequence-number))"))
            (middle
              ()
              (field ((type . "uint16_t") (name . "sequence")))))))
      (xsl:when
        ((test . "self::eventcopy|self::errorcopy"))
        (typedef
          ((newname . "{xcb:xcb-prefix(@name)}{$suffix}"))
          (xsl:attribute
            ((name . "oldname"))
            (xsl:call-template
              ((name . "canonical-type-name"))
              (xsl:with-param
                ((name . "type") (select . "@ref"))))
            (xsl:value-of ((select . "$suffix"))))))))
  (xsl:template
    ((match . "typedef") (mode . "pass1"))
    (typedef
      ()
      (xsl:attribute
        ((name . "oldname"))
        (xsl:call-template
          ((name . "canonical-type-name"))
          (xsl:with-param
            ((name . "type") (select . "@oldname"))))
        (xsl:text () "_t"))
      (xsl:attribute
        ((name . "newname"))
        (xsl:call-template
          ((name . "canonical-type-name"))
          (xsl:with-param
            ((name . "type") (select . "@newname"))))
        (xsl:text () "_t")))
    (iterator ((ref . "{xcb:xcb-prefix(@newname)}")))
    (iterator-functions
      ((ref . "{xcb:xcb-prefix(@newname)}"))))
  (xsl:template
    ((match . "enum") (mode . "pass1"))
    (enum ((name . "{xcb:xcb-prefix(@name)}_t"))
          (xsl:for-each
            ((select . "item"))
            (item ((name .
                         "{translate(xcb:xcb-prefix(concat(../@name, concat('_', @name))), $lcase, $ucase)}"))
                  (xsl:copy-of ((select . "*")))))))
  (xsl:template
    ((match . "pad") (mode . "field"))
    (xsl:copy-of ((select . "."))))
  (xsl:template
    ((match . "field|exprfield") (mode . "field"))
    (xsl:copy
      ()
      (xsl:attribute
        ((name . "type"))
        (xsl:call-template
          ((name . "canonical-type-name")))
        (xsl:text () "_t"))
      (xsl:attribute
        ((name . "name"))
        (xsl:call-template
          ((name . "canonical-var-name"))))
      (xsl:copy-of ((select . "*")))))
  (xsl:template
    ((match . "list") (mode . "field"))
    (xsl:variable
      ((name . "type"))
      (xsl:call-template
        ((name . "canonical-type-name")))
      (xsl:text () "_t"))
    (list ((type . "{$type}"))
          (xsl:attribute
            ((name . "name"))
            (xsl:call-template
              ((name . "canonical-var-name"))))
          (xsl:if
            ((test .
                   "not(parent::request) and node()                     and not(.//*[not(self::value or self::op)])"))
            (xsl:attribute ((name . "fixed")) "true"))
          (xsl:if
            ((test . "not(node())"))
            (xsl:choose
              ()
              (xsl:when
                ((test . "parent::request"))
                (fieldref
                  ()
                  (xsl:value-of
                    ((select . "concat(@name, '_len')")))))
              (xsl:when
                ((test . "parent::reply"))
                (op ((op . "/"))
                    (op ((op . "<<"))
                        (fieldref () "length")
                        (value () "2"))
                    (function-call
                      ((name . "sizeof"))
                      (param () (xsl:value-of ((select . "$type")))))))
              (xsl:otherwise
                ()
                (xsl:message
                  ((terminate . "yes"))
                  "Encountered a list with no length expresssion outside a"
                  " request or reply."))))
          (xsl:copy-of ((select . "*")))))
  (xsl:template
    ((match . "valueparam") (mode . "field"))
    (field ()
           (xsl:attribute
             ((name . "type"))
             (xsl:call-template
               ((name . "canonical-type-name"))
               (xsl:with-param
                 ((name . "type") (select . "@value-mask-type"))))
             (xsl:text () "_t"))
           (xsl:attribute
             ((name . "name"))
             (xsl:call-template
               ((name . "canonical-var-name"))
               (xsl:with-param
                 ((name . "name") (select . "@value-mask-name"))))))
    (list ((type . "uint32_t"))
          (xsl:attribute
            ((name . "name"))
            (xsl:call-template
              ((name . "canonical-var-name"))
              (xsl:with-param
                ((name . "name") (select . "@value-list-name")))))
          (function-call
            ((name . "xcb_popcount"))
            (param ()
                   (fieldref
                     ()
                     (xsl:call-template
                       ((name . "canonical-var-name"))
                       (xsl:with-param
                         ((name . "name") (select . "@value-mask-name")))))))))
  (xsl:template
    ((match . "field") (mode . "param"))
    (field ()
           (xsl:attribute
             ((name . "type"))
             (xsl:call-template
               ((name . "canonical-type-name")))
             (xsl:text () "_t"))
           (xsl:attribute
             ((name . "name"))
             (xsl:call-template
               ((name . "canonical-var-name"))))))
  (xsl:template
    ((match . "list") (mode . "param"))
    (xsl:if
      ((test . "not(node())"))
      (field ((type . "uint32_t") (name . "{@name}_len"))))
    (field ()
           (xsl:variable
             ((name . "ctype"))
             (xsl:call-template
               ((name . "canonical-type-name"))))
           (xsl:attribute
             ((name . "type"))
             (xsl:text () "const ")
             (xsl:call-template
               ((name . "canonical-type-name")))
             (xsl:if
               ((test .
                      "not($ctype='char') and not($ctype='void')"))
               (xsl:text () "_t"))
             (xsl:text () " *"))
           (xsl:attribute
             ((name . "name"))
             (xsl:call-template
               ((name . "canonical-var-name"))))))
  (xsl:template
    ((match . "valueparam") (mode . "param"))
    (field ()
           (xsl:attribute
             ((name . "type"))
             (xsl:call-template
               ((name . "canonical-type-name"))
               (xsl:with-param
                 ((name . "type") (select . "@value-mask-type"))))
             (xsl:text () "_t"))
           (xsl:attribute
             ((name . "name"))
             (xsl:call-template
               ((name . "canonical-var-name"))
               (xsl:with-param
                 ((name . "name") (select . "@value-mask-name"))))))
    (field ((type . "const uint32_t *"))
           (xsl:attribute
             ((name . "name"))
             (xsl:call-template
               ((name . "canonical-var-name"))
               (xsl:with-param
                 ((name . "name") (select . "@value-list-name")))))))
  (xsl:variable
    ((name . "result-rtf"))
    (xsl:apply-templates
      ((select . "$pass1/*") (mode . "pass2"))))
  (xsl:variable
    ((name . "result")
     (select . "e:node-set($result-rtf)")))
  (xsl:template
    ((match . "xcb") (mode . "pass2"))
    (xcb ()
         (xsl:copy-of ((select . "@*")))
         (xsl:apply-templates
           ((mode . "pass2")
            (select
              .
              "constant|enum|struct|typedef|iterator")))
         (xsl:apply-templates
           ((mode . "pass2")
            (select . "function|iterator-functions")))))
  (xsl:template
    ((match . "*") (mode . "pass2"))
    (xsl:copy
      ()
      (xsl:copy-of ((select . "@*")))
      (xsl:apply-templates ((mode . "pass2")))))
  (xsl:template
    ((match . "struct") (mode . "pass2"))
    (xsl:if
      ((test . "@kind='union' and list[not(@fixed)]"))
      (xsl:message
        ((terminate . "yes"))
        "Unions must be fixed length."))
    (struct
      ((name . "{@name}"))
      (xsl:if
        ((test . "@kind"))
        (xsl:attribute
          ((name . "kind"))
          (xsl:value-of ((select . "@kind")))))
      (xsl:copy-of
        ((select
           .
           "node()[not(self::middle)                    and position() < 3]")))
      (xsl:if
        ((test .
               "middle and (count(*[not(self::middle)]) < 2)"))
        (pad ((bytes . "{2 - count(*[not(self::middle)])}"))))
      (xsl:copy-of ((select . "middle/*")))
      (xsl:copy-of
        ((select
           .
           "node()[not(self::middle) and (position() > 2)]")))))
  (xsl:template
    ((match . "do-request") (mode . "pass2"))
    (xsl:variable
      ((name . "struct")
       (select
         .
         "$pass1/xcb/struct[@name=current()/@ref]")))
    (xsl:variable
      ((name . "num-parts")
       (select . "(1+count($struct/list))*2")))
    (l ()
       "static const xcb_protocol_request_t xcb_req = {")
    (indent
      ()
      (l ()
         "/* count */ "
         (xsl:value-of ((select . "$num-parts")))
         ",")
      (l ()
         "/* ext */ "
         (xsl:choose
           ()
           (xsl:when
             ((test . "$ext"))
             (xsl:text () "&")
             (xsl:value-of ((select . "xcb:xcb-prefix()")))
             (xsl:text () "_id"))
           (xsl:otherwise () "0"))
         ",")
      (l ()
         "/* opcode */ "
         (xsl:value-of ((select . "@opcode")))
         ",")
      (l ()
         "/* isvoid */ "
         (xsl:value-of
           ((select . "1-boolean(@has-reply)")))))
    (l () "};")
    (l ())
    (l ()
       "struct iovec xcb_parts["
       (xsl:value-of ((select . "$num-parts+2")))
       "];")
    (l ()
       (xsl:value-of ((select . "../@type")))
       " xcb_ret;")
    (l ()
       (xsl:value-of ((select . "@ref")))
       " xcb_out;")
    (l ())
    (xsl:if
      ((test .
             "not ($ext) and not($struct//*[(self::field or self::exprfield or self::pad)                                                 and not(boolean(@no-assign))])"))
      (l () "xcb_out.pad0 = 0;"))
    (xsl:apply-templates
      ((select
         .
         "$struct//*[(self::field or self::exprfield or self::pad)                                             and not(boolean(@no-assign))]")
       (mode . "assign")))
    (l ())
    (l ()
       "xcb_parts[2].iov_base = (char *) "
       "&"
       "xcb_out;")
    (l () "xcb_parts[2].iov_len = sizeof(xcb_out);")
    (l () "xcb_parts[3].iov_base = 0;")
    (l ()
       "xcb_parts[3].iov_len = -xcb_parts[2].iov_len "
       "&"
       " 3;")
    (xsl:for-each
      ((select . "$struct/list"))
      (l ()
         "xcb_parts["
         (xsl:value-of ((select . "2 + position() * 2")))
         "].iov_base = (char *) "
         (xsl:value-of ((select . "@name")))
         ";")
      (l ()
         "xcb_parts["
         (xsl:value-of ((select . "2 + position() * 2")))
         "].iov_len = "
         (xsl:apply-templates
           ((mode . "output-expression")))
         (xsl:if
           ((test . "not(@type = 'void_t')"))
           (xsl:text () " * sizeof(")
           (xsl:choose
             ()
             (xsl:when
               ((test . "@type='char_t'"))
               (xsl:text () "char"))
             (xsl:otherwise
               ()
               (xsl:value-of ((select . "@type")))))
           (xsl:text () ")"))
         ";")
      (l ()
         "xcb_parts["
         (xsl:value-of ((select . "3 + position() * 2")))
         "].iov_base = 0;")
      (l ()
         "xcb_parts["
         (xsl:value-of ((select . "3 + position() * 2")))
         "].iov_len = -xcb_parts["
         (xsl:value-of ((select . "2 + position() * 2")))
         "].iov_len "
         "&"
         " 3;"))
    (l ()
       "xcb_ret.sequence = xcb_send_request(c, "
       (xsl:choose
         ()
         (xsl:when
           ((test . "@checked='true'"))
           "XCB_REQUEST_CHECKED")
         (xsl:otherwise () "0"))
       ", xcb_parts + 2, "
       "&"
       "xcb_req);")
    (l () "return xcb_ret;"))
  (xsl:template
    ((match . "field") (mode . "assign"))
    (l ()
       (xsl:text () "xcb_out.")
       (xsl:value-of ((select . "@name")))
       (xsl:text () " = ")
       (xsl:value-of ((select . "@name")))
       (xsl:text () ";")))
  (xsl:template
    ((match . "exprfield") (mode . "assign"))
    (l ()
       (xsl:text () "xcb_out.")
       (xsl:value-of ((select . "@name")))
       (xsl:text () " = ")
       (xsl:apply-templates
         ((mode . "output-expression")))
       (xsl:text () ";")))
  (xsl:template
    ((match . "pad") (mode . "assign"))
    (xsl:variable
      ((name . "padnum"))
      (xsl:number ()))
    (l ()
       (xsl:choose
         ()
         (xsl:when
           ((test . "@bytes = 1"))
           "xcb_out.pad"
           (xsl:value-of ((select . "$padnum - 1")))
           " = 0;")
         (xsl:otherwise
           ()
           "memset(xcb_out.pad"
           (xsl:value-of ((select . "$padnum - 1")))
           ", 0, "
           (xsl:value-of ((select . "@bytes")))
           ");"))))
  (xsl:template
    ((match . "iterator") (mode . "pass2"))
    (struct
      ((name . "{@ref}_iterator_t"))
      (field ((type . "{@ref}_t *") (name . "data")))
      (field ((type . "int") (name . "rem")))
      (field ((type . "int") (name . "index")))))
  (xsl:template
    ((match . "iterator-functions") (mode . "pass2"))
    (xsl:variable ((name . "ref") (select . "@ref")))
    (xsl:variable
      ((name . "kind") (select . "@kind")))
    (xsl:variable
      ((name . "struct")
       (select
         .
         "$pass1/xcb/struct[@name=concat($ref, $kind, '_t')]")))
    (xsl:variable
      ((name . "nextfields-rtf"))
      (nextfield () "R + 1")
      (xsl:for-each
        ((select . "$struct/list[not(@fixed)]"))
        (xsl:choose
          ()
          (xsl:when
            ((test . "substring(@type, 1, 3) = 'xcb'"))
            (nextfield
              ()
              (xsl:value-of
                ((select
                   .
                   "substring(@type, 1, string-length(@type)-2)")))
              "_end("
              (xsl:value-of ((select . "$ref")))
              "_"
              (xsl:value-of ((select . "string(@name)")))
              "_iterator(R))"))
          (xsl:otherwise
            ()
            (nextfield
              ()
              (xsl:value-of ((select . "$ref")))
              "_"
              (xsl:value-of ((select . "string(@name)")))
              "_end(R)")))))
    (xsl:variable
      ((name . "nextfields")
       (select . "e:node-set($nextfields-rtf)")))
    (xsl:for-each
      ((select . "$struct/list[not(@fixed)]"))
      (xsl:variable
        ((name . "number")
         (select
           .
           "1+count(preceding-sibling::list[not(@fixed)])")))
      (xsl:variable
        ((name . "nextfield")
         (select . "$nextfields/nextfield[$number]")))
      (xsl:variable
        ((name . "is-first")
         (select
           .
           "not(preceding-sibling::list[not(@fixed)])")))
      (xsl:variable
        ((name . "field-name") (select . "@name")))
      (xsl:variable
        ((name . "is-variable")
         (select
           .
           "$pass1/xcb/struct[@name=current()/@type]/list                             or document($search-path)/xcb                                /struct[concat(xcb:xcb-prefix(@name), '_t')                                        = current()/@type]                                /*[self::valueparam                                   or self::list[.//*[not(self::value                                                          or self::op)]]]")))
      (xsl:if
        ((test . "not($is-variable)"))
        (function
          ((type . "{xcb:get-char-void(@type)} *")
           (name . "{$ref}_{xcb:lowercase($field-name)}"))
          (field ((type . "const {$ref}{$kind}_t *") (name . "R")))
          (xsl:choose
            ()
            (xsl:when
              ((test . "$is-first"))
              (l ()
                 "return ("
                 (xsl:value-of
                   ((select . "xcb:get-char-void(@type)")))
                 " *) "
                 "("
                 (xsl:value-of ((select . "$nextfield")))
                 ");"))
            (xsl:otherwise
              ()
              (l ()
                 "xcb_generic_iterator_t prev = "
                 (xsl:value-of ((select . "$nextfield")))
                 ";")
              (l ()
                 "return ("
                 (xsl:value-of
                   ((select . "xcb:get-char-void(@type)")))
                 " *) "
                 "((char *) prev.data + XCB_TYPE_PAD("
                 (xsl:value-of
                   ((select . "xcb:get-char-void(@type)")))
                 ", prev.index));")))))
      (function
        ((type . "int")
         (name .
               "{$ref}_{xcb:lowercase($field-name)}_length"))
        (field ((type . "const {$ref}{$kind}_t *") (name . "R")))
        (l ()
           "return "
           (xsl:apply-templates
             ((mode . "output-expression"))
             (xsl:with-param
               ((name . "field-prefix") (select . "'R->'"))))
           ";"))
      (xsl:choose
        ()
        (xsl:when
          ((test . "substring(@type, 1, 3) = 'xcb'"))
          (function
            ((type .
                   "{substring(@type, 1, string-length(@type)-2)}_iterator_t")
             (name .
                   "{$ref}_{xcb:lowercase($field-name)}_iterator"))
            (field ((type . "const {$ref}{$kind}_t *") (name . "R")))
            (l ()
               (xsl:value-of
                 ((select
                    .
                    "substring(@type, 1, string-length(@type)-2)")))
               "_iterator_t i;")
            (xsl:choose
              ()
              (xsl:when
                ((test . "$is-first"))
                (l ()
                   "i.data = ("
                   (xsl:value-of ((select . "@type")))
                   " *) "
                   "("
                   (xsl:value-of ((select . "$nextfield")))
                   ");"))
              (xsl:otherwise
                ()
                (l ()
                   "xcb_generic_iterator_t prev = "
                   (xsl:value-of ((select . "$nextfield")))
                   ";")
                (l ()
                   "i.data = ("
                   (xsl:value-of ((select . "@type")))
                   " *) "
                   "((char *) prev.data + XCB_TYPE_PAD("
                   (xsl:value-of ((select . "@type")))
                   ", prev.index));")))
            (l ()
               "i.rem = "
               (xsl:apply-templates
                 ((mode . "output-expression"))
                 (xsl:with-param
                   ((name . "field-prefix") (select . "'R->'"))))
               ";")
            (l () "i.index = (char *) i.data - (char *) R;")
            (l () "return i;")))
        (xsl:otherwise
          ()
          (xsl:variable
            ((name . "cast"))
            (xsl:choose
              ()
              (xsl:when ((test . "@type='void'")) "char")
              (xsl:otherwise
                ()
                (xsl:value-of ((select . "@type"))))))
          (function
            ((type . "xcb_generic_iterator_t")
             (name .
                   "{$ref}_{xcb:lowercase($field-name)}_end"))
            (field ((type . "const {$ref}{$kind}_t *") (name . "R")))
            (l () "xcb_generic_iterator_t i;")
            (xsl:choose
              ()
              (xsl:when
                ((test . "$is-first"))
                (l ()
                   "i.data = (("
                   (xsl:value-of
                     ((select . "xcb:remove-void($cast)")))
                   " *) "
                   "("
                   (xsl:value-of ((select . "$nextfield")))
                   ")) + ("
                   (xsl:apply-templates
                     ((mode . "output-expression"))
                     (xsl:with-param
                       ((name . "field-prefix") (select . "'R->'"))))
                   ");"))
              (xsl:otherwise
                ()
                (l ()
                   "xcb_generic_iterator_t child = "
                   (xsl:value-of ((select . "$nextfield")))
                   ";")
                (l ()
                   "i.data = (("
                   (xsl:value-of
                     ((select . "xcb:get-char-void($cast)")))
                   " *) "
                   "child.data) + ("
                   (xsl:apply-templates
                     ((mode . "output-expression"))
                     (xsl:with-param
                       ((name . "field-prefix") (select . "'R->'"))))
                   ");")))
            (l () "i.rem = 0;")
            (l () "i.index = (char *) i.data - (char *) R;")
            (l () "return i;")))))
    (xsl:if
      ((test . "not($kind)"))
      (function
        ((type . "void") (name . "{$ref}_next"))
        (field ((type . "{$ref}_iterator_t *") (name . "i")))
        (xsl:choose
          ()
          (xsl:when
            ((test . "$struct/list[not(@fixed)]"))
            (l ()
               (xsl:value-of ((select . "$ref")))
               "_t *R = i->data;")
            (l ()
               "xcb_generic_iterator_t child = "
               (xsl:value-of
                 ((select . "$nextfields/nextfield[last()]")))
               ";")
            (l () "--i->rem;")
            (l ()
               "i->data = ("
               (xsl:value-of ((select . "$ref")))
               "_t *) child.data;")
            (l () "i->index = child.index;"))
          (xsl:otherwise
            ()
            (l () "--i->rem;")
            (l () "++i->data;")
            (l ()
               "i->index += sizeof("
               (xsl:value-of ((select . "$ref")))
               "_t);"))))
      (function
        ((type . "xcb_generic_iterator_t")
         (name . "{$ref}_end"))
        (field ((type . "{$ref}_iterator_t") (name . "i")))
        (l () "xcb_generic_iterator_t ret;")
        (xsl:choose
          ()
          (xsl:when
            ((test . "$struct/list[not(@fixed)]"))
            (l () "while(i.rem > 0)")
            (indent
              ()
              (l ()
                 (xsl:value-of ((select . "$ref")))
                 "_next("
                 "&"
                 "i);"))
            (l () "ret.data = i.data;")
            (l () "ret.rem = i.rem;")
            (l () "ret.index = i.index;"))
          (xsl:otherwise
            ()
            (l () "ret.data = i.data + i.rem;")
            (l ()
               "ret.index = i.index + ((char *) ret.data - (char *) i.data);")
            (l () "ret.rem = 0;")))
        (l () "return ret;"))))
  (xsl:template
    ((match . "/"))
    (xsl:if
      ((test . "not(function-available('e:node-set'))"))
      (xsl:message
        ((terminate . "yes"))
        "Error: This stylesheet requires the EXSL node-set extension."))
    (xsl:if
      ((test . "not($h) and not($c)"))
      (xsl:message
        ((terminate . "yes"))
        "Error: Parameter \"mode\" must be \"header\" or \"source\"."))
    (xsl:apply-templates
      ((select . "$result/*") (mode . "output"))))
  (xsl:template
    ((match . "xcb") (mode . "output"))
    (xsl:variable
      ((name . "guard"))
      "__"
      (xsl:value-of ((select . "$ucase-header")))
      "_H")
    (xsl:text
      ()
      "/*"
      " * This file generated automatically from ")
    (xsl:value-of ((select . "$header")))
    (xsl:text
      ()
      ".xml by c-client.xsl using XSLT."
      " * Edit at your peril."
      " */")
    (xsl:if
      ((test . "$h"))
      (xsl:text () "/**" " * @defgroup XCB_")
      (xsl:value-of ((select . "$ext")))
      (xsl:text () "_API XCB ")
      (xsl:value-of ((select . "$ext")))
      (xsl:text () " API" " * @brief ")
      (xsl:value-of ((select . "$ext")))
      (xsl:text () " XCB Protocol Implementation.")
      (xsl:text () " * @{" " **/")
      (xsl:text () "#ifndef ")
      (xsl:value-of ((select . "$guard")))
      (xsl:text () "#define ")
      (xsl:value-of ((select . "$guard")))
      (xsl:text ())
      "#include \"xcb.h\""
      (xsl:for-each
        ((select . "$root/xcb/import"))
        (xsl:text () "#include \"")
        (xsl:value-of ((select . ".")))
        (xsl:text () ".h\""))
      (xsl:text ()))
    (xsl:if
      ((test . "$h"))
      (xsl:choose
        ()
        (xsl:when
          ((test . "string($ext)"))
          (xsl:text () "#define XCB_")
          (xsl:value-of
            ((select . "translate($ext, $lcase, $ucase)")))
          (xsl:text () "_MAJOR_VERSION ")
          (xsl:value-of ((select . "/xcb/@major-version")))
          (xsl:text ())
          (xsl:text () "#define XCB_")
          (xsl:value-of
            ((select . "translate($ext, $lcase, $ucase)")))
          (xsl:text () "_MINOR_VERSION ")
          (xsl:value-of ((select . "/xcb/@minor-version")))
          (xsl:text ()))))
    (xsl:if
      ((test . "$c"))
      (xsl:if
        ((test . "$need-string-h"))
        "#include "
        "<"
        "string.h"
        ">")
      (xsl:text
        ()
        "#include "
        "<"
        "assert.h"
        ">"
        "#include \"xcbext.h\""
        "#include \"")
      (xsl:value-of ((select . "$header")))
      (xsl:text () ".h\""))
    (xsl:apply-templates ((mode . "output")))
    (xsl:if
      ((test . "$h"))
      (xsl:text () "#endif" "/**" " * @}" " */")))
  (xsl:template
    ((match . "constant") (mode . "output"))
    (xsl:choose
      ()
      (xsl:when
        ((test . "@type = 'number'"))
        (xsl:if
          ((test . "$h"))
          (xsl:text () "/** Opcode for ")
          (xsl:value-of ((select . "@name")))
          (xsl:text () ". */")
          (xsl:text () "#define ")
          (xsl:value-of
            ((select . "translate(@name, $lcase, $ucase)")))
          (xsl:text ())
          (xsl:value-of ((select . "@value")))
          (xsl:text ())))
      (xsl:when
        ((test . "@type = 'string'"))
        (xsl:if ((test . "$h")) (xsl:text () "extern "))
        (xsl:text () "const char ")
        (xsl:value-of ((select . "@name")))
        (xsl:text () "[]")
        (xsl:if
          ((test . "$c"))
          (xsl:text () " = \"")
          (xsl:value-of ((select . "@value")))
          (xsl:text () "\""))
        (xsl:text () ";"))
      (xsl:otherwise
        ()
        (xsl:if ((test . "$h")) (xsl:text () "extern "))
        (xsl:call-template ((name . "type-and-name")))
        (xsl:if
          ((test . "$c"))
          (xsl:text () " = ")
          (xsl:value-of ((select . "@value"))))
        (xsl:text () ";"))))
  (xsl:template
    ((match . "typedef") (mode . "output"))
    (xsl:if
      ((test . "$h"))
      (xsl:text () "typedef ")
      (xsl:value-of
        ((select . "xcb:get-char-void(@oldname)")))
      (xsl:text ())
      (xsl:value-of ((select . "@newname")))
      (xsl:text () ";")))
  (xsl:template
    ((match . "struct") (mode . "output"))
    (xsl:if
      ((test . "$h"))
      (xsl:variable
        ((name . "type-lengths"))
        (xsl:call-template
          ((name . "type-lengths"))
          (xsl:with-param
            ((name . "items") (select . "field/@type")))))
      (xsl:text () "/**" " * @brief ")
      (xsl:value-of ((select . "@name")))
      (xsl:text () " **/")
      (xsl:text () "typedef ")
      (xsl:if ((test . "not(@kind)")) "struct")
      (xsl:value-of ((select . "@kind")))
      (xsl:text ())
      (xsl:value-of ((select . "@name")))
      (xsl:text () " {")
      (xsl:for-each
        ((select . "exprfield|field|list[@fixed]|pad"))
        (xsl:text ())
        (xsl:apply-templates
          ((select . "."))
          (xsl:with-param
            ((name . "type-lengths")
             (select . "$type-lengths"))))
        (xsl:text () "; /**" "<")
        (xsl:text () " */"))
      (xsl:text () "} ")
      (xsl:value-of ((select . "@name")))
      (xsl:text () ";")))
  (xsl:template
    ((match . "enum") (mode . "output"))
    (xsl:if
      ((test . "$h"))
      (xsl:text () "typedef enum ")
      (xsl:value-of ((select . "@name")))
      (xsl:text () " {")
      (xsl:call-template
        ((name . "list"))
        (xsl:with-param
          ((name . "separator"))
          (xsl:text () ","))
        (xsl:with-param
          ((name . "items"))
          (xsl:for-each
            ((select . "item"))
            (item ()
                  (xsl:value-of ((select . "@name")))
                  (xsl:if
                    ((test . "node()"))
                    (xsl:text () " = ")
                    (xsl:apply-templates
                      ((mode . "output-expression"))))))))
      (xsl:text () "} ")
      (xsl:value-of ((select . "@name")))
      (xsl:text () ";")))
  (xsl:template
    ((match . "function") (mode . "output"))
    (xsl:variable
      ((name . "decl-open")
       (select . "concat(@name, ' (')")))
    (xsl:variable
      ((name . "type-lengths"))
      (xsl:call-template
        ((name . "type-lengths"))
        (xsl:with-param
          ((name . "items") (select . "field/@type")))))
    "/*****************************************************************************"
    " **"
    " ** "
    (xsl:value-of ((select . "@type")))
    (xsl:text ())
    (xsl:value-of ((select . "@name")))
    " ** "
    (xsl:call-template
      ((name . "list"))
      (xsl:with-param
        ((name . "items"))
        (xsl:for-each
          ((select . "field"))
          (item ()
                (xsl:text () " ** @param ")
                (xsl:apply-templates
                  ((select . "."))
                  (xsl:with-param
                    ((name . "type-lengths")
                     (select . "$type-lengths"))))))))
    " ** @returns "
    (xsl:value-of ((select . "@type")))
    " **"
    " *****************************************************************************/"
    (xsl:value-of ((select . "@type")))
    (xsl:text ())
    (xsl:value-of ((select . "$decl-open")))
    (xsl:call-template
      ((name . "list"))
      (xsl:with-param
        ((name . "separator"))
        (xsl:text () ",")
        (xsl:call-template
          ((name . "repeat"))
          (xsl:with-param
            ((name . "count")
             (select . "string-length($decl-open)")))))
      (xsl:with-param
        ((name . "items"))
        (xsl:for-each
          ((select . "field"))
          (item ()
                (xsl:apply-templates
                  ((select . "."))
                  (xsl:with-param
                    ((name . "type-lengths")
                     (select . "$type-lengths"))))
                (xsl:text () "  /**" "<" " */")))))
    (xsl:text () ")")
    (xsl:if ((test . "$h")) (xsl:text () ";"))
    (xsl:if
      ((test . "$c"))
      (xsl:text () "{")
      (xsl:apply-templates
        ((select . "l|indent") (mode . "function-body"))
        (xsl:with-param
          ((name . "indent") (select . "$indent-string"))))
      (xsl:text () "}")))
  (xsl:template
    ((match . "l") (mode . "function-body"))
    (xsl:param ((name . "indent")))
    (xsl:value-of ((select . "concat($indent, .)")))
    (xsl:text ()))
  (xsl:template
    ((match . "indent") (mode . "function-body"))
    (xsl:param ((name . "indent")))
    (xsl:apply-templates
      ((select . "l|indent") (mode . "function-body"))
      (xsl:with-param
        ((name . "indent")
         (select . "concat($indent, $indent-string)")))))
  (xsl:template
    ((match . "value") (mode . "output-expression"))
    (xsl:value-of ((select . "."))))
  (xsl:template
    ((match . "fieldref")
     (mode . "output-expression"))
    (xsl:param ((name . "field-prefix")))
    (xsl:value-of
      ((select . "concat($field-prefix, .)"))))
  (xsl:template
    ((match . "op") (mode . "output-expression"))
    (xsl:param ((name . "field-prefix")))
    (xsl:text () "(")
    (xsl:apply-templates
      ((select . "node()[1]")
       (mode . "output-expression"))
      (xsl:with-param
        ((name . "field-prefix")
         (select . "$field-prefix"))))
    (xsl:text ())
    (xsl:value-of ((select . "@op")))
    (xsl:text ())
    (xsl:apply-templates
      ((select . "node()[2]")
       (mode . "output-expression"))
      (xsl:with-param
        ((name . "field-prefix")
         (select . "$field-prefix"))))
    (xsl:text () ")"))
  (xsl:template
    ((match . "bit") (mode . "output-expression"))
    (xsl:text () "(1 " "<" "<")
    (xsl:value-of ((select . ".")))
    (xsl:text () ")"))
  (xsl:template
    ((match . "function-call")
     (mode . "output-expression"))
    (xsl:param ((name . "field-prefix")))
    (xsl:value-of ((select . "@name")))
    (xsl:text () "(")
    (xsl:call-template
      ((name . "list"))
      (xsl:with-param
        ((name . "separator") (select . "', '")))
      (xsl:with-param
        ((name . "items"))
        (xsl:for-each
          ((select . "param"))
          (item ()
                (xsl:apply-templates
                  ((mode . "output-expression"))
                  (xsl:with-param
                    ((name . "field-prefix")
                     (select . "$field-prefix"))))))))
    (xsl:text () ")"))
  (xsl:template
    ((match . "*") (mode . "output-expression"))
    (xsl:message
      ((terminate . "yes"))
      (xsl:text () "Invalid element in expression: ")
      (xsl:value-of ((select . "name()")))))
  (xsl:template
    ((match . "field|exprfield"))
    (xsl:param
      ((name . "type-lengths") (select . "0")))
    (xsl:call-template
      ((name . "type-and-name"))
      (xsl:with-param
        ((name . "type-lengths")
         (select . "$type-lengths")))))
  (xsl:template
    ((match . "list[@fixed]"))
    (xsl:param
      ((name . "type-lengths") (select . "0")))
    (xsl:call-template
      ((name . "type-and-name"))
      (xsl:with-param
        ((name . "type-lengths")
         (select . "$type-lengths"))))
    (xsl:text () "[")
    (xsl:apply-templates
      ((mode . "output-expression")))
    (xsl:text () "]"))
  (xsl:template
    ((match . "pad"))
    (xsl:param
      ((name . "type-lengths") (select . "0")))
    (xsl:variable
      ((name . "padnum"))
      (xsl:number ()))
    (xsl:call-template
      ((name . "type-and-name"))
      (xsl:with-param
        ((name . "type") (select . "'uint8_t'")))
      (xsl:with-param
        ((name . "name"))
        (xsl:text () "pad")
        (xsl:value-of ((select . "$padnum - 1"))))
      (xsl:with-param
        ((name . "type-lengths")
         (select . "$type-lengths"))))
    (xsl:if
      ((test . "@bytes > 1"))
      (xsl:text () "[")
      (xsl:value-of ((select . "@bytes")))
      (xsl:text () "]")))
  (xsl:template
    ((name . "type-and-name"))
    (xsl:param ((name . "type") (select . "@type")))
    (xsl:param ((name . "name") (select . "@name")))
    (xsl:param
      ((name . "type-lengths"))
      (max-type-length () "0")
      (max-suffix-length () "0"))
    (xsl:variable
      ((name . "type-lengths-ns")
       (select . "e:node-set($type-lengths)")))
    (xsl:variable
      ((name . "min-type-length")
       (select . "$type-lengths-ns/max-type-length")))
    (xsl:variable
      ((name . "min-suffix-length")
       (select . "$type-lengths-ns/max-suffix-length")))
    (xsl:variable
      ((name . "base-type"))
      (xsl:choose
        ()
        (xsl:when
          ((test . "contains($type, ' *')"))
          (xsl:value-of
            ((select . "substring-before($type, ' *')"))))
        (xsl:otherwise
          ()
          (xsl:value-of ((select . "$type"))))))
    (xsl:variable
      ((name . "suffix"))
      (xsl:if
        ((test . "contains($type, ' *')"))
        (xsl:text () "*")
        (xsl:value-of
          ((select . "substring-after($type, ' *')")))))
    (xsl:value-of ((select . "$base-type")))
    (xsl:if
      ((test .
             "string-length($base-type) < $min-type-length"))
      (xsl:call-template
        ((name . "repeat"))
        (xsl:with-param
          ((name . "count")
           (select
             .
             "$min-type-length                                              - string-length($base-type)")))))
    (xsl:text ())
    (xsl:if
      ((test .
             "string-length($suffix) < $min-suffix-length"))
      (xsl:call-template
        ((name . "repeat"))
        (xsl:with-param
          ((name . "count")
           (select
             .
             "$min-suffix-length                                              - string-length($suffix)")))))
    (xsl:value-of ((select . "$suffix")))
    (xsl:value-of ((select . "$name"))))
  (xsl:template
    ((name . "list"))
    (xsl:param ((name . "separator")))
    (xsl:param ((name . "items")))
    (xsl:for-each
      ((select . "e:node-set($items)/*"))
      (xsl:value-of ((select . ".")))
      (xsl:if
        ((test . "not(position() = last())"))
        (xsl:value-of ((select . "$separator"))))))
  (xsl:template
    ((name . "repeat"))
    (xsl:param ((name . "str") (select . "' '")))
    (xsl:param ((name . "count")))
    (xsl:if
      ((test . "$count > 0"))
      (xsl:value-of ((select . "$str")))
      (xsl:call-template
        ((name . "repeat"))
        (xsl:with-param
          ((name . "str") (select . "$str")))
        (xsl:with-param
          ((name . "count") (select . "$count - 1"))))))
  (xsl:template
    ((name . "type-lengths"))
    (xsl:param ((name . "items")))
    (xsl:variable
      ((name . "type-lengths-rtf"))
      (xsl:for-each
        ((select . "$items"))
        (item ()
              (xsl:choose
                ()
                (xsl:when
                  ((test . "contains(., ' *')"))
                  (xsl:value-of
                    ((select
                       .
                       "string-length(                                     substring-before(., ' *'))"))))
                (xsl:otherwise
                  ()
                  (xsl:value-of ((select . "string-length(.)"))))))))
    (xsl:variable
      ((name . "suffix-lengths-rtf"))
      (xsl:for-each
        ((select . "$items"))
        (item ()
              (xsl:choose
                ()
                (xsl:when
                  ((test . "contains(., ' *')"))
                  (xsl:value-of
                    ((select
                       .
                       "string-length(substring-after(., ' *'))                                     + 1"))))
                (xsl:otherwise () (xsl:text () "0"))))))
    (max-type-length
      ()
      (xsl:call-template
        ((name . "max"))
        (xsl:with-param
          ((name . "items")
           (select . "e:node-set($type-lengths-rtf)/*")))))
    (max-suffix-length
      ()
      (xsl:call-template
        ((name . "max"))
        (xsl:with-param
          ((name . "items")
           (select . "e:node-set($suffix-lengths-rtf)/*"))))))
  (xsl:template
    ((name . "max"))
    (xsl:param ((name . "items")))
    (xsl:choose
      ()
      (xsl:when
        ((test . "count($items) = 0"))
        (xsl:text () "0"))
      (xsl:otherwise
        ()
        (xsl:variable
          ((name . "head") (select . "number($items[1])")))
        (xsl:variable
          ((name . "tail-max"))
          (xsl:call-template
            ((name . "max"))
            (xsl:with-param
              ((name . "items")
               (select . "$items[position() > 1]")))))
        (xsl:choose
          ()
          (xsl:when
            ((test . "$head > number($tail-max)"))
            (xsl:value-of ((select . "$head"))))
          (xsl:otherwise
            ()
            (xsl:value-of ((select . "$tail-max")))))))))
