;;; xsb-util.scm

(define-module (X-ok-C-no xsb util)
  #:export (-connect
            -disconnect
            -make-fwt
            -merge-fwts)
  #:use-module ((ice-9 accumulate) #:select (accumulator/one-only))
  #:use-module ((ice-9 lol) #:select (lol-map))
  #:use-module ((database binconv) #:select (integer->integer-byte-string
                                             integer-byte-string->integer))
  #:use-module ((ttn-do zz sys linux-gnu) #:select (iovec
                                                    writev
                                                    readv)))

;;;---------------------------------------------------------------------------
;;; connection

(define (-connect . opt)
  (let* ((d (or (and (not (null? opt))
                     (car opt))
                (getenv "DISPLAY")))
         (colon (string-index d #\:))
         (host (and (not (zero? colon))
                    (substring d 0 colon)))
         (display (substring d (1+ colon)
                             (or (string-index d #\.)
                                 (string-length d)))))

    (define (tcp)
      (error "NOT YET IMPLEMENTED: (-connect tcp)"))

    (define (unix)
      (let ((name (simple-format #f "/tmp/.X11-unix/X~A" display))
            (sock (socket PF_UNIX SOCK_STREAM 0)))
        (connect sock AF_UNIX name)
        sock))

    ;; fixme: screen
    (cond (host (tcp))
          (else (unix)))))

(define (-disconnect conn)
  (shutdown conn 2))

;;;---------------------------------------------------------------------------
;;; fixed width trees

(define (-make-fwt . specs)
  (let ((total 0)
        (count 0)
        (names (accumulator/one-only))
        (puts (accumulator/one-only))
        (gets (accumulator/one-only))
        (bufs (accumulator/one-only)))

    (define (grok name bytes)
      (or (memq bytes '(1 2 4 8))
          (error "invalid spec" spec))
      (set! total (+ total bytes))
      (set! count (1+ count))
      (names name)
      (let ((buf (make-string bytes #\nul)))
        (puts (case bytes
                ((1)
                 (lambda (n) (string-set! buf 0 (integer->char n))))
                ((2 4 8)
                 (lambda (n) (integer->integer-byte-string n bytes #f #t buf)))))
        (gets (case bytes
                ((1)
                 (lambda () (char->integer (string-ref buf 0))))
                ((2 4 8)
                 (lambda () (integer-byte-string->integer buf #f #t)))))
        (bufs buf)))

    (for-each (lambda (spec)
                (apply-to-args spec grok))
              specs)
    (set! names (names))
    (set! bufs (bufs))
    (set! puts (map cons names (puts)))
    (set! gets (map cons names (gets)))

    (list (list total)
          (list count)
          (list (iovec bufs #t))
          (list (lambda (name value)
                  ((assq-ref puts name) value)))
          (list (lambda (name)
                  ((assq-ref gets name)))))))

(define (lol-transpose lol)
  (if (null? lol)
      lol
      (let* ((width (length (car lol)))
             (cols (map (lambda ignored
                          (accumulator/one-only))
                        (iota width))))
        (for-each (lambda (row)
                    (for-each (lambda (col value)
                                (col value))
                              cols row))
                  lol)
        (map (lambda (col)
               (col))
             cols))))

(define (-merge-fwts . fwts)
  (map (lambda (x)
         (apply append x))
       (lol-transpose fwts)))

;;; xsb-util.scm ends here
